
class MarketTypes:

    MATCH_ODDS = 'MATCH_ODDS'
    DRAW_NO_BET = 'DRAW_NO_BET'
    FULLTIME_SCORELINE = 'CORRECT_SCORE'
    HALFTIME_SCORELINE = 'HALF_TIME_SCORE'
    BOTH_TEAMS_TO_SCORE = 'BOTH_TEAMS_TO_SCORE'
    OVER_UNDER_05 = 'OVER_UNDER_05'
    OVER_UNDER_15 = 'OVER_UNDER_15'
    OVER_UNDER_25 = 'OVER_UNDER_25'
    OVER_UNDER_35 = 'OVER_UNDER_35'
    OVER_UNDER_45 = 'OVER_UNDER_45'
    OVER_UNDER_55 = 'OVER_UNDER_55'
    HALFTIME_FULLTIME = 'HALF_TIME_FULL_TIME'

    ALL = [
        MATCH_ODDS, DRAW_NO_BET, FULLTIME_SCORELINE, HALFTIME_SCORELINE, BOTH_TEAMS_TO_SCORE,
        OVER_UNDER_05, OVER_UNDER_15, OVER_UNDER_25, OVER_UNDER_35, OVER_UNDER_45, OVER_UNDER_55,
        HALFTIME_FULLTIME
    ]
